#!/bin/bash

[ -d /.host -a -d /.in -a -d /.out ] || {
	echo >&2 "This script is intended to run inside of hasher."
	exit 1
}

[ "$RPM_INSTALL_NAME" = "kdump-tools-checkinstall" ] || {
	echo >&2 "This script is intended to run inside of checkinstall."
	exit 1
}

cd /tmp
set -euxo pipefail

vm_prepare()
{
	sed -i  -e '/#KDUMP_CMDLINE_REMOVE/s/#//' \
		-e '/^KDUMP_CMDLINE_REMOVE/s/"$/ quiet"/'\
		/etc/sysconfig/kdump-tools
	vm-create-image /tmp/i
}

cause_panic()
{
	df -h
	make-initrd
	kdump-config show
	kdump-config load
	kdump-config status
	sync
	kdump-config test
	sleep 1
	echo 1 >/proc/sys/kernel/sysrq
	echo c >/proc/sysrq-trigger
}

analyze()
{
        df -h
        find /var/crash -ls
        kdumpctl list
        file /var/crash/*/kdump
        file /var/crash/*/kdump | grep "Kdump compressed dump"
        kdumpctl dmesg | grep "Kdump: loaded"
        find /var/crash -type f -perm /77 -ls -exec false {} +
}

vm_panic()
{
	timeout 100 vm-run --rootfs=/tmp/i --kvm=only --append='crashkernel=256M' "$0" cause_panic |& tee /tmp/log \
	|| echo 'Failure is expected because of crash.'
	grep "sysrq: Trigger a crash" /tmp/log
	grep "Kernel panic - not syncing: sysrq triggered crash" /tmp/log
	grep "Kdump: loaded" /tmp/log
}

vm_analyze()
{
	timeout 100 vm-run --rootfs=/tmp/i --kvm=only "$0" analyze
}

if [ $# -eq 0 ]; then
	[ -w /dev/kvm ] || {
		echo >&2 "/dev/kvm is needed to run this script."
		exit 1
	}
	vm_prepare
	vm_panic
	vm_analyze
else
	"$@"
fi
